@@ -1,6 +1,6 @@
 import json
 import requests
-from typing import Any, TypeVar, Union
+from typing import Any, TypeVar, Optional
 from .credentials import Credentials
 from .authentication import verify_credentials
 
@@ -11,9 +11,9 @@ def execute_request(
     credentials: Credentials,
     method: str,
     path: str,
-    body: Union[Any, None] = None,
-    params: Union[Any, None] = None,
-    file: Union[str, None] = None,
+    body: Optional[Any] = None,
+    params: Optional[Any] = None,
+    file: Optional[str] = None,
 ) -> Any:
   verified_credentials = verify_credentials(credentials)
   headers = {
