@@ -74,6 +74,15 @@ class UpdatesInput(TypedDict):
   updateUntil: UpdateUntilInput
 
 
+class Perspective(Enum):
+  RESIDUAL = 'Residual'
+  RESIDUAL_CHANGE = 'ResidualChange'
+  FLUCTUATION = 'Fluctuation'
+  FLUCTUATION_CHANGE = 'FluctuationChange'
+  IMBALANCE = 'Imbalance'
+  IMBALANCE_CHANGE = 'ImbalanceChange'
+
+
 class Data(TypedDict):
   version: Id
   rows: Union[RelativeRange, List[Range]]
@@ -93,7 +102,7 @@ class DetectData(TypedDict):
 
 
 class DomainSpecificsInput(TypedDict):
-  perspective: str
+  perspective: Perspective
   sensitivity: float
   minSensitivity: float
   maxSensitivity: float
@@ -119,12 +128,23 @@ class AnomalousBehaviorModelInput(TypedDict):
   detectionIntervals: List[DetectionIntervalsInput]
 
 
+class RebuildType(Enum):
+  DOMAIN_SPECIFICS = 'DomainSpecifics'
+  ANOMALOUS_BEHAVIOR_MODEL = 'AnomalousBehaviorModel'
+  ALL = 'All'
+
+
 class Configuration(TypedDict):
   domainSpecifics: List[DomainSpecificsInput]
   normalBehaviorModel: NormalBehaviorModelInput
   anomalousBehaviorModel: AnomalousBehaviorModelInput
 
 
+class RebuildConfiguration(TypedDict, total=False):
+  domainSpecifics: List[DomainSpecificsInput]
+  rebuildType: RebuildType
+
+
 class AnomalyDetectionJobConfiguration(TypedDict, total=False):
   name: str
   useCase: Id
@@ -137,6 +157,12 @@ class AnomalyDetectionDetectConfiguration(TypedDict, total=False):
   data: DetectData
 
 
+class AnomalyDetectionRebuildModelConfiguration(TypedDict, total=False):
+  name: str
+  configuration: RebuildConfiguration
+  data: DetectData
+
+
 class BuildModelResponse(TypedDict):
   id: str
   expectedResultsTableSize: float
