@@ -5,10 +5,12 @@ from typing import List, Union, Callable, Optional
 from tim.core.credentials import Credentials
 from tim.core.api import execute_request
 from tim.types import Logs, Status, ExecuteResponse
-from .types import CSVSeparator, DatasetStatusResponse, DatasetListPayload, Dataset, DatasetListVersion, UploadCsvResponse, UploadCSVConfiguration
+from .types import CSVSeparator, DatasetStatusResponse, DatasetListPayload, Dataset, DatasetListVersion, UpdateDatasetConfiguration, UpdateDatasetResponse, UploadDatasetResponse, UploadDatasetConfiguration
 
 
-def __is_valid_csv_configuration(configuration: UploadCSVConfiguration) -> bool:
+def __is_valid_csv_configuration(
+    configuration: Union[UploadDatasetConfiguration, UpdateDatasetConfiguration]
+) -> bool:
   if configuration is None:
     return True
 
@@ -17,15 +19,15 @@ def __is_valid_csv_configuration(configuration: UploadCSVConfiguration) -> bool:
   return True
 
 
-def upload_csv(
+def upload_dataset(
     credentials: Credentials,
     dataset: DataFrame,
-    configuration: UploadCSVConfiguration,
-) -> UploadCsvResponse:
+    configuration: UploadDatasetConfiguration,
+) -> UploadDatasetResponse:
   if not __is_valid_csv_configuration(configuration):
     raise ValueError("Invalid configuration input.")
 
-  conf_with_csv_separator: UploadCSVConfiguration = copy(configuration)
+  conf_with_csv_separator: UploadDatasetConfiguration = copy(configuration)
   conf_with_csv_separator["csvSeparator"
                          ] = CSVSeparator.SEMICOLON.value  # pyright: reportTypedDictNotRequiredAccess=false
 
@@ -38,6 +40,28 @@ def upload_csv(
   )
 
 
+def update_dataset(
+    credentials: Credentials,
+    id: str,
+    dataset: DataFrame,
+    configuration: UpdateDatasetConfiguration,
+) -> UpdateDatasetResponse:
+  if not __is_valid_csv_configuration(configuration):
+    raise ValueError("Invalid configuration input.")
+
+  conf_with_csv_separator: UpdateDatasetConfiguration = copy(configuration)
+  conf_with_csv_separator["csvSeparator"
+                         ] = CSVSeparator.SEMICOLON.value  # pyright: reportTypedDictNotRequiredAccess=false
+
+  return execute_request(
+      credentials=credentials,
+      method="patch",
+      path=f"/datasets/{id}/csv",
+      body=conf_with_csv_separator,
+      file=dataset.to_csv(sep=conf_with_csv_separator["csvSeparator"], index=False),
+  )
+
+
 def get_version_status(credentials: Credentials, id: str, version_id: str) -> DatasetStatusResponse:
   return execute_request(
       credentials=credentials,
@@ -88,8 +112,8 @@ def get_datasets(
     credentials: Credentials,
     offset: int,
     limit: int,
-    workspace_id: Union[str, None] = None,
-    sort: Union[str, None] = None
+    workspace_id: Optional[str] = None,
+    sort: Optional[str] = None
 ) -> List[Dataset]:
 
   payload = DatasetListPayload(offset=offset, limit=limit)
