@@ -1,12 +1,12 @@
 from io import StringIO
 import pandas as pd
 from time import sleep
-from typing import Optional, List, Union, Callable
+from typing import Optional, List, Callable
 from tim.core import Credentials
 from tim.core.api import execute_request
 from tim.core.credentials import Credentials
 from tim.types import ExecuteResponse, Logs, Status, StatusResponse
-from .types import BuildModelResponse, ForecastAccuracies, ForecastModelResult, ForecastJobConfiguration, ForecastMetadata, ForecastTableRequestPayload
+from .types import BuildModelResponse, ForecastAccuracies, ForecastModelResult, ForecastJobConfiguration, ForecastMetadata, ForecastTableRequestPayload, ForecastingPredictConfiguration, ForecastingRebuildModelConfiguration
 
 
 def build_model(credentials: Credentials, job_configuration: ForecastJobConfiguration) -> BuildModelResponse:
@@ -15,6 +15,29 @@ def build_model(credentials: Credentials, job_configuration: ForecastJobConfigur
   )
 
 
+def predict(
+    credentials: Credentials, parent_job_id: str, job_configuration: Optional[ForecastingPredictConfiguration]
+) -> BuildModelResponse:
+  return execute_request(
+      credentials=credentials,
+      method='post',
+      path=f'/forecast-jobs/{parent_job_id}/predict',
+      body=job_configuration
+  )
+
+
+def rebuild_model(
+    credentials: Credentials, parent_job_id: str,
+    job_configuration: Optional[ForecastingRebuildModelConfiguration]
+) -> BuildModelResponse:
+  return execute_request(
+      credentials=credentials,
+      method='post',
+      path=f'/forecast-jobs/{parent_job_id}/rebuild-model',
+      body=job_configuration
+  )
+
+
 def execute(credentials: Credentials, id: str) -> ExecuteResponse:
   return execute_request(credentials=credentials, method='post', path=f'/forecast-jobs/{id}/execute')
 
@@ -29,16 +52,16 @@ def get_forecast(credentials: Credentials, id: str) -> ForecastMetadata:
 
 def get_forecasting_jobs(
     credentials: Credentials,
-    offset: Union[int, None] = None,
-    limit: Union[int, None] = None,
-    sort: Union[str, None] = None,
-    experiment_id: Union[str, None] = None,
-    use_case_id: Union[str, None] = None,
-    type: Union[str, None] = None,
-    status: Union[str, None] = None,
-    parent_id: Union[str, None] = None,
-    from_datetime: Union[str, None] = None,
-    to_datetime: Union[str, None] = None
+    offset: Optional[int] = None,
+    limit: Optional[int] = None,
+    sort: Optional[str] = None,
+    experiment_id: Optional[str] = None,
+    use_case_id: Optional[str] = None,
+    type: Optional[str] = None,
+    status: Optional[str] = None,
+    parent_id: Optional[str] = None,
+    from_datetime: Optional[str] = None,
+    to_datetime: Optional[str] = None
 ) -> List[ForecastMetadata]:
   payload = {
       "experimentId": experiment_id,
