@@ -84,6 +84,26 @@ class Aggregation(Enum):
   MAXIMUM = 'Maximum'
 
 
+class OffsetLimitType(Enum):
+  EXPLICIT = 'Explicit'
+
+
+class OffsetLimit(TypedDict):
+  type: OffsetLimitType
+  value: int
+
+
+class RebuildingType(Enum):
+  NEW_SITUATIONS = 'NewSituations'
+  ALL = 'All'
+  OLDER_THAN = 'OlderThan'
+
+
+class RebuildingPolicy(TypedDict, total=False):
+  type: RebuildingType
+  time: RelativeRange
+
+
 class Configuration(TypedDict, total=False):
   predictionTo: RelativeRange
   predictionFrom: RelativeRange
@@ -92,6 +112,8 @@ class Configuration(TypedDict, total=False):
   maxModelComplexity: int
   features: List[Feature]
   dailyCycle: bool
+  allowOffsets: bool
+  offsetLimit: OffsetLimit
   memoryLimitCheck: bool
   predictionIntervals: float
   predictionBoundaries: PredictionBoundaries
@@ -99,6 +121,29 @@ class Configuration(TypedDict, total=False):
   backtest: Backtest
 
 
+class PredictConfiguration(TypedDict, total=False):
+  predictionTo: RelativeRange
+  predictionFrom: RelativeRange
+  predictionBoundaries: PredictionBoundaries
+  rollingWindow: RelativeRange
+
+
+class RebuildConfiguration(TypedDict, total=False):
+  predictionTo: RelativeRange
+  predictionFrom: RelativeRange
+  modelQuality: ModelQuality
+  normalization: bool
+  maxModelComplexity: int
+  features: List[Feature]
+  allowOffsets: bool
+  offsetLimit: OffsetLimit
+  memoryLimitCheck: bool
+  rebuildingPolicy: RebuildingPolicy
+  predictionBoundaries: PredictionBoundaries
+  rollingWindow: RelativeRange
+  backtest: Backtest
+
+
 class Data(TypedDict, total=False):
   version: Id
   inSampleRows: Union[RelativeRange, List[Range]]
@@ -111,6 +156,20 @@ class Data(TypedDict, total=False):
   aggregation: Aggregation
 
 
+class PredictData(TypedDict, total=False):
+  version: Id
+  outOfSampleRows: Union[RelativeRange, List[Range]]
+  imputation: Imputation
+
+
+class RebuildData(TypedDict, total=False):
+  version: Id
+  inSampleRows: Union[RelativeRange, List[Range]]
+  outOfSampleRows: Union[RelativeRange, List[Range]]
+  imputation: Imputation
+  columns: List[Union[str, int]]
+
+
 class ForecastJobConfiguration(TypedDict, total=False):
   name: str
   useCase: Id
@@ -124,6 +183,18 @@ class BuildForecastingModelConfiguration(TypedDict, total=False):
   data: Data
 
 
+class ForecastingPredictConfiguration(TypedDict, total=False):
+  name: str
+  configuration: PredictConfiguration
+  data: PredictData
+
+
+class ForecastingRebuildModelConfiguration(TypedDict, total=False):
+  name: str
+  configuration: RebuildConfiguration
+  data: RebuildData
+
+
 class BuildModelResponse(TypedDict):
   id: str
   expectedResultsTableSize: float
@@ -144,6 +215,7 @@ class ErrorMeasures(TypedDict):
 class ForecastJobType(Enum):
   BUILD_MODEL = 'build-model'
   REBUILD_MODEL = 'rebuild-model'
+  PREDICT = 'predict'
   RCA = 'rca'
 
 
@@ -152,7 +224,7 @@ class JobLoad(Enum):
   HEAVY = 'Heavy'
 
 
-class ForecastMetadata(NamedTuple):
+class ForecastMetadata(TypedDict):
   id: str
   name: str
   type: ForecastJobType
