@@ -1,10 +1,11 @@
 from typing_extensions import TypedDict
+from tim.types import Id
 
 
 class UseCaseConfiguration(TypedDict):
   name: str
-  workspaceId: str
-  datasetId: str
+  workspace: Id
+  dataset: Id
 
 
 class CreateUseCaseResponse(TypedDict):
@@ -17,8 +18,8 @@ class CreateUseCaseResponse(TypedDict):
   businessObjective: str
   businessKpi: str
   accuracyImpact: int
-  workspaceId: str
-  datasetId: str
+  workspace: Id
+  dataset: Id
   isFavorite: bool
   createdAt: str
   createdBy: str
