@@ -1,15 +1,15 @@
 from tim.core.credentials import Credentials
 from tim.core.api import execute_request
 from .types import WorkspaceListPayload, Workspace
-from typing import List, Union
+from typing import List, Optional
 
 
 def get_workspaces(
     credentials: Credentials,
     offset: int,
     limit: int,
-    user_group_id: Union[str, None] = None,
-    sort: Union[str, None] = None
+    user_group_id: Optional[str] = None,
+    sort: Optional[str] = None
 ) -> List[Workspace]:
   payload = WorkspaceListPayload(offset=offset, limit=limit)
   if user_group_id: payload['userGroupId'] = user_group_id
