@@ -1,15 +1,15 @@
 from datetime import datetime
 from typing import List, Union, Callable, Optional
 from tim.data_sources.anomaly_detection.anomaly_detection import get_anomaly_detection_job_status, get_anomaly_detection_jobs
-from tim.data_sources.anomaly_detection import build_anomaly_detection_model, detect, execute_anomaly_detection, get_anomaly_detection, get_anomaly_detection_logs, get_anomaly_detection_model_results, get_anomaly_detection_table_results, poll_anomaly_detection_status, delete_anomaly_detection
+from tim.data_sources.anomaly_detection import build_anomaly_detection_model, detect, rebuild_anomaly_detection_model, execute_anomaly_detection, get_anomaly_detection, get_anomaly_detection_logs, get_anomaly_detection_model_results, get_anomaly_detection_table_results, poll_anomaly_detection_status, delete_anomaly_detection
 from pandas import DataFrame
 from tim.core.credentials import Credentials
-from tim.data_sources.forecast.types import CleanForecastResponse, ForecastJobConfiguration, ForecastMetadata, ForecastResultsResponse, BuildForecastingModelConfiguration
-from tim.data_sources.anomaly_detection.types import AnomalyDetectionType, AnomalyDetectionJobConfiguration, AnomalyDetectionDetectConfiguration, AnomalyDetection, AnomalyDetectionResultsResponse, BuildAnomalyDetectionModelConfiguration
+from tim.data_sources.forecast.types import CleanForecastResponse, ForecastJobConfiguration, ForecastJobType, ForecastMetadata, ForecastResultsResponse, BuildForecastingModelConfiguration, ForecastingPredictConfiguration, ForecastingRebuildModelConfiguration
+from tim.data_sources.anomaly_detection.types import AnomalyDetectionRebuildModelConfiguration, AnomalyDetectionType, AnomalyDetectionJobConfiguration, AnomalyDetectionDetectConfiguration, AnomalyDetection, AnomalyDetectionResultsResponse, BuildAnomalyDetectionModelConfiguration
 from tim.data_sources.workspace.types import Workspace
-from tim.data_sources.dataset.types import Dataset, DatasetListVersion, DatasetStatusResponse
-from tim.data_sources.dataset import get_dataset, delete_dataset, get_datasets, get_dataset_versions, get_dataset_logs, upload_csv, poll_dataset_version_status, UploadDatasetResponse, UploadCSVConfiguration
-from tim.data_sources.forecast import build_forecasting_model, execute, get_forecast, get_forecast_accuracies_result, get_forecast_logs, get_forecast_model_results, get_forecast_table_results, poll_forecast_status, get_status, delete_forecast, get_forecasting_jobs
+from tim.data_sources.dataset.types import Dataset, DatasetListVersion, DatasetStatusResponse, UpdateDatasetConfiguration, UpdateDatasetResponse
+from tim.data_sources.dataset import get_dataset, delete_dataset, get_datasets, get_dataset_versions, get_dataset_logs, upload_dataset, update_dataset, poll_dataset_version_status, UploadDatasetResponse, UploadDatasetResultsResponse, UploadDatasetConfiguration
+from tim.data_sources.forecast import build_forecasting_model, rebuild_forecasting_model, predict, execute, get_forecast, get_forecast_accuracies_result, get_forecast_logs, get_forecast_model_results, get_forecast_table_results, poll_forecast_status, get_status, delete_forecast, get_forecasting_jobs
 from tim.data_sources.use_case import create_use_case, UseCaseConfiguration
 from tim.data_sources.workspace import get_workspaces
 from tim.types import ExecuteResponse, Status, Id, StatusResponse
@@ -31,9 +31,10 @@ class Tim:
   def upload_dataset(
       self,
       dataset: DataFrame,
-      configuration: UploadCSVConfiguration = UploadCSVConfiguration(),
+      configuration: UploadDatasetConfiguration = UploadDatasetConfiguration(),
+      wait_to_finish: bool = True,
       handle_status_poll: Optional[Callable[[DatasetStatusResponse], None]] = None
-  ) -> UploadDatasetResponse:
+  ) -> Union[UploadDatasetResultsResponse, UploadDatasetResponse]:
     """Upload a dataset to the TIM repository
 
         Parameters
@@ -44,6 +45,9 @@ class Tim:
         	Metadata of the dataset, Optional
           Available keys are: timestampFormat, timestampColumn, decimalSeparator, name, description and samplingPeriod
         	The value of samplingPeriod is a Dict containing the keys baseUnit and value
+      wait_to_finish : bool, Optional
+        Wait for the dataset to be uploaded before returning
+        If set to False, the function will return once the dataset upload process has started
       handle_status_poll: Callable, Optional
         A callback function to poll for the status and progress of the dataset upload
 
@@ -53,7 +57,9 @@ class Tim:
         	Dict when successful; None when unsuccessful
         logs : list of Dict
         """
-    upload_response = upload_csv(self.__credentials, dataset, configuration)
+    upload_response = upload_dataset(self.__credentials, dataset, configuration)
+    if wait_to_finish is False: return upload_response
+
     id = upload_response['id']
 
     status_result = poll_dataset_version_status(
@@ -69,10 +75,58 @@ class Tim:
 
     logs = get_dataset_logs(self.__credentials, id)
 
-    return UploadDatasetResponse(metadata, logs)
+    return UploadDatasetResultsResponse(metadata, logs)
+
+  def update_dataset(
+      self,
+      dataset_id: str,
+      dataset_version: DataFrame,
+      configuration: UpdateDatasetConfiguration = UpdateDatasetConfiguration(),
+      wait_to_finish: bool = True,
+      handle_status_poll: Optional[Callable[[DatasetStatusResponse], None]] = None
+  ) -> Union[UploadDatasetResultsResponse, UpdateDatasetResponse]:
+    """Update a dataset in the TIM repository by uploading a new version
+
+        Parameters
+        ----------
+        dataset_id: str
+          The ID of the dataset to update
+        dataset_version : DataFrame
+        	The dataset containing time-series data
+        configuration: Dict
+        	Metadata of the dataset, Optional
+          Available keys are: timestampFormat, timestampColumn, decimalSeparator
+      wait_to_finish : bool, Optional
+        Wait for the dataset version to be uploaded before returning
+        If set to False, the function will return once the dataset version upload process has started
+      handle_status_poll: Callable, Optional
+        A callback function to poll for the status and progress of the dataset version upload
+
+        Returns
+        -------
+        dataset_metadata : Dict | None
+        	Dict when successful; None when unsuccessful
+        logs : list of Dict
+        """
+    update_response = update_dataset(self.__credentials, dataset_id, dataset_version, configuration)
+    if wait_to_finish is False: return update_response
+
+    id = update_response['version']['id']
+
+    status_result = poll_dataset_version_status(
+        credentials=self.__credentials, id=dataset_id, version_id=id, handle_status_poll=handle_status_poll
+    )
+
+    metadata = None
+    if Status(status_result['status']).value != Status.FAILED.value:
+      metadata = get_dataset(self.__credentials, dataset_id)
+
+    logs = get_dataset_logs(self.__credentials, dataset_id)
+
+    return UploadDatasetResultsResponse(metadata, logs)
 
   def build_forecasting_model(
-      self, dataset_id: str, job_configuration: Union[BuildForecastingModelConfiguration, None] = None
+      self, dataset_id: str, job_configuration: Optional[BuildForecastingModelConfiguration] = None
   ) -> str:
     """Create a forecast job in the workspace the dataset is connected to (the default workspace)
 
@@ -140,56 +194,10 @@ class Tim:
     poll_forecast_status(self.__credentials, forecast_job_id, handle_status_poll)
     return self.get_forecast_results(forecast_job_id)
 
-  def get_forecast_results(self, forecast_job_id: str) -> ForecastResultsResponse:
-    """Retrieve the results of a forecast job
-
-    Parameters
-    ----------
-    forecast_job_id : str
-        The ID of a forecast job
-
-    Returns
-    -------
-    metadata : Dict | None
-      Dict when successful; None when unsuccessful
-    model_result : Dict | None
-      Dict when successful; None when unsuccessful
-    table_result : DataFrame | None
-      Dict when successful; None when unsuccessful
-    accuracies : Dict | None
-      Dict when successful; None when unsuccessful
-    logs : list of Dict
-    """
-    metadata = model_result = table_result = accuracies = None
-    status = get_status(self.__credentials, forecast_job_id)
-    logs = get_forecast_logs(self.__credentials, forecast_job_id)
-
-    if Status(status['status']).value != Status.FAILED.value:
-      metadata = get_forecast(self.__credentials, forecast_job_id)
-      model_result = get_forecast_model_results(self.__credentials, forecast_job_id)
-      table_result = get_forecast_table_results(self.__credentials, forecast_job_id)
-      accuracies = get_forecast_accuracies_result(self.__credentials, forecast_job_id)
-
-    return ForecastResultsResponse(metadata, model_result, table_result, accuracies, logs)
-
-  def delete_forecast(self, forecast_job_id: str) -> ExecuteResponse:
-    """Delete a forecasting job
-
-    Parameters
-    ----------
-    forecast_job_id : str
-        ID of the forecasting job to delete
-    Returns
-    -------
-    message : Dict
-        Available keys: message (str) and code (str)
-    """
-    return delete_forecast(self.__credentials, forecast_job_id)
-
   def build_forecasting_model_and_execute(
       self,
       dataset_id: str,
-      job_configuration: Union[BuildForecastingModelConfiguration, None] = None,
+      job_configuration: Optional[BuildForecastingModelConfiguration] = None,
       wait_to_finish: bool = True,
       handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
   ) -> Union[ForecastResultsResponse, ExecuteResponse]:
@@ -223,11 +231,175 @@ class Tim:
     id = self.build_forecasting_model(dataset_id, job_configuration)
     return self.execute_forecast(id, wait_to_finish, handle_status_poll)
 
+  def create_forecast(
+      self, parent_job_id: str, job_configuration: Optional[ForecastingPredictConfiguration] = None
+  ) -> str:
+    """Create a forecasting job using the same model as the parent forecasting job
+
+    Parameters
+    ----------
+    parent_job_id : str
+      The ID of a parent forecasting job
+    job_configuration : ForecastingPredictConfiguration
+      TIM Engine forecasting configuration
+      Available keys are : name, configuration, data
+
+    Returns
+    -------
+    id : str
+    """
+    return predict(
+        credentials=self.credentials, parent_job_id=parent_job_id, job_configuration=job_configuration
+    )['id']
+
+  def create_forecast_and_execute(
+      self,
+      parent_job_id: str,
+      job_configuration: Optional[ForecastingPredictConfiguration] = None,
+      wait_to_finish: bool = True,
+      handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
+  ) -> Union[ForecastResultsResponse, ExecuteResponse]:
+    """Create a forecasting job using the same model as the parent forecasting job and execute it
+
+    Parameters
+    ----------
+    parent_job_id : str
+        The ID of a parent forecasting job
+    job_configuration : ForecastingPredictConfiguration
+        TIM Engine forecasting configuration
+        Available keys are : name, configuration, data
+    wait_to_finish : bool, Optional
+        Wait for all results to be calculated before returning
+        If set to False, the function will return once the job has started the execution process
+    handle_status_poll: Callable, Optional
+        A callback function to poll for the status and progress of the forecasting job execution
+
+    Returns
+    -------
+    metadata : Dict | None
+        Dict when successful; None when unsuccessful
+    model_result : Dict | None
+        Dict when successful; None when unsuccessful
+    table_result : DataFrame | None
+        DataFrame when successful; None when unsuccessful
+    accuracies : Dict | None
+        Dict when successful; None when unsuccessful
+    logs : list of Dict
+    """
+    id = self.create_forecast(parent_job_id=parent_job_id, job_configuration=job_configuration)
+    return self.execute_forecast(id, wait_to_finish, handle_status_poll)
+
+  def rebuild_forecasting_model(
+      self, parent_job_id: str, job_configuration: Optional[ForecastingRebuildModelConfiguration] = None
+  ) -> str:
+    """Create a forecast job to rebuild the model(s) of the parent forecasting job
+
+    Parameters
+    ----------
+    parent_job_id : str
+        The ID of a parent forecasting job
+    job_configuration : ForecastingRebuildModelConfiguration, Optional
+        TIM Engine model rebuilding and forecasting configuration, by default None
+        Available keys are: name, configuration, data
+
+    Returns
+    -------
+    id : str
+    """
+    return rebuild_forecasting_model(
+        credentials=self.credentials, parent_job_id=parent_job_id, job_configuration=job_configuration
+    )['id']
+
+  def rebuild_forecasting_model_and_execute(
+      self,
+      parent_job_id: str,
+      job_configuration: Optional[ForecastingRebuildModelConfiguration] = None,
+      wait_to_finish: bool = True,
+      handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
+  ) -> Union[ForecastResultsResponse, ExecuteResponse]:
+    """Create a forecast job to rebuild the model(s) of the parent forecasting job and execute it
+
+    Parameters
+    ----------
+    parent_job_id : str
+        The ID of a parent forecasting job
+    job_configuration : ForecastingRebuildModelConfiguration, Optional
+        TIM Engine model rebuilding and forecasting configuration, by default None
+        Available keys are: name, configuration, data
+    wait_to_finish : bool, Optional
+      Wait for all results to be calculated before returning
+      If set to False, the function will return once the job has started the execution process
+    handle_status_poll: Callable, Optional
+      A callback function to poll for the status and progress of the forecasting job execution
+
+    Returns
+    -------
+    metadata : Dict | None
+      Dict when successful; None when unsuccessful
+    model_result : Dict | None
+      Dict when successful; None when unsuccessful
+    table_result : DataFrame | None
+      DataFrame when successful; None when unsuccessful
+    accuracies : Dict | None
+      Dict when successful; None when unsuccessful
+    logs : list of Dict
+    """
+    id = self.rebuild_forecasting_model(parent_job_id=parent_job_id, job_configuration=job_configuration)
+    return self.execute_forecast(id, wait_to_finish, handle_status_poll)
+
+  def get_forecast_results(self, forecast_job_id: str) -> ForecastResultsResponse:
+    """Retrieve the results of a forecast job
+
+    Parameters
+    ----------
+    forecast_job_id : str
+      The ID of a forecast job
+
+    Returns
+    -------
+    metadata : Dict | None
+      Dict when successful; None when unsuccessful
+    model_result : Dict | None
+      Dict when successful; None when unsuccessful
+    table_result : DataFrame | None
+      Dict when successful; None when unsuccessful
+    accuracies : Dict | None
+      Dict when successful; None when unsuccessful
+    logs : list of Dict
+    """
+    metadata = model_result = table_result = accuracies = None
+    status = get_status(self.__credentials, forecast_job_id)
+    logs = get_forecast_logs(self.__credentials, forecast_job_id)
+
+    if Status(status['status']).value != Status.FAILED.value:
+      metadata = get_forecast(self.__credentials, forecast_job_id)
+      table_result = get_forecast_table_results(self.__credentials, forecast_job_id)
+      accuracies = get_forecast_accuracies_result(self.__credentials, forecast_job_id)
+      is_predict_job = ForecastJobType(metadata['type']).value == ForecastJobType.PREDICT.value
+      job_id = metadata['parentJob']['id'] if is_predict_job else forecast_job_id
+      model_result = get_forecast_model_results(self.__credentials, job_id)
+
+    return ForecastResultsResponse(metadata, model_result, table_result, accuracies, logs)
+
+  def delete_forecast(self, forecast_job_id: str) -> ExecuteResponse:
+    """Delete a forecasting job
+
+    Parameters
+    ----------
+    forecast_job_id : str
+        ID of the forecasting job to delete
+    Returns
+    -------
+    message : Dict
+        Available keys: message (str) and code (str)
+    """
+    return delete_forecast(self.__credentials, forecast_job_id)
+
   def clean_forecast(
       self,
       dataset: DataFrame,
-      dataset_configuration: UploadCSVConfiguration = UploadCSVConfiguration(),
-      job_configuration: Union[BuildForecastingModelConfiguration, None] = None,
+      dataset_configuration: UploadDatasetConfiguration = UploadDatasetConfiguration(),
+      job_configuration: Optional[BuildForecastingModelConfiguration] = None,
       handle_dataset_upload_status_poll: Optional[Callable[[DatasetStatusResponse], None]] = None,
       handle_forecast_status_poll: Optional[Callable[[StatusResponse], None]] = None
   ) -> Union[CleanForecastResponse, ExecuteResponse]:
@@ -265,38 +437,40 @@ class Tim:
     if job_configuration and "data" in job_configuration and "dataVersion" in job_configuration["data"]:
       raise ValueError("'dataVersion' is not a valid key in in data in job_configuration for this function.")
 
-    upload_response, dataset_logs = self.upload_dataset(
-        dataset, dataset_configuration, handle_dataset_upload_status_poll
+    upload_response = self.upload_dataset(
+        dataset, dataset_configuration, True, handle_dataset_upload_status_poll
     )
 
-    if upload_response:
-      execute_response = self.build_forecasting_model_and_execute(
-          upload_response['id'], job_configuration, True, handle_forecast_status_poll
-      )
-      self.delete_dataset(upload_response['id'])
-
-      if isinstance(execute_response, ForecastResultsResponse):
-        return CleanForecastResponse(
-            metadata=execute_response.metadata,
-            model_result=execute_response.model_result,
-            table_result=execute_response.table_result,
-            accuracies=execute_response.accuracies,
-            forecast_logs=execute_response.logs,
-            dataset_logs=dataset_logs
+    if isinstance(upload_response, UploadDatasetResultsResponse):
+      if upload_response.dataset:
+        execute_response = self.build_forecasting_model_and_execute(
+            upload_response.dataset['id'], job_configuration, True, handle_forecast_status_poll
         )
-      raise ValueError("Internal error. Please contact support.")
+        self.delete_dataset(upload_response.dataset['id'])
 
-    return CleanForecastResponse(
-        metadata=None,
-        model_result=None,
-        table_result=None,
-        accuracies=None,
-        forecast_logs=None,
-        dataset_logs=dataset_logs
-    )
+        if isinstance(execute_response, ForecastResultsResponse):
+          return CleanForecastResponse(
+              metadata=execute_response.metadata,
+              model_result=execute_response.model_result,
+              table_result=execute_response.table_result,
+              accuracies=execute_response.accuracies,
+              forecast_logs=execute_response.logs,
+              dataset_logs=upload_response.logs
+          )
+        raise ValueError("Internal error. Please contact support.")
+
+      return CleanForecastResponse(
+          metadata=None,
+          model_result=None,
+          table_result=None,
+          accuracies=None,
+          forecast_logs=None,
+          dataset_logs=upload_response.logs
+      )
+    raise ValueError("Internal error. Please contact support.")
 
   def build_anomaly_detection_model(
-      self, dataset_id: str, job_configuration: Union[BuildAnomalyDetectionModelConfiguration, None] = None
+      self, dataset_id: str, job_configuration: Optional[BuildAnomalyDetectionModelConfiguration] = None
   ) -> str:
     """Create an anomaly detection job in the workspace the dataset is connected to (default workspace)
 
@@ -365,7 +539,7 @@ class Tim:
   def build_anomaly_detection_model_and_execute(
       self,
       dataset_id: str,
-      job_configuration: Union[BuildAnomalyDetectionModelConfiguration, None] = None,
+      job_configuration: Optional[BuildAnomalyDetectionModelConfiguration] = None,
       wait_to_finish: bool = True,
       handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
   ) -> Union[AnomalyDetectionResultsResponse, ExecuteResponse]:
@@ -398,7 +572,7 @@ class Tim:
     return self.execute_anomaly_detection(id, wait_to_finish, handle_status_poll)
 
   def create_anomaly_detection(
-      self, parent_job_id: str, job_configuration: Union[AnomalyDetectionDetectConfiguration, None] = None
+      self, parent_job_id: str, job_configuration: Optional[AnomalyDetectionDetectConfiguration] = None
   ) -> str:
     """Create an anomaly detection job using the same model as the parent anomaly detection job
 
@@ -421,7 +595,7 @@ class Tim:
   def create_anomaly_detection_and_execute(
       self,
       parent_job_id: str,
-      job_configuration: Union[AnomalyDetectionDetectConfiguration, None] = None,
+      job_configuration: Optional[AnomalyDetectionDetectConfiguration] = None,
       wait_to_finish: bool = True,
       handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
   ) -> Union[AnomalyDetectionResultsResponse, ExecuteResponse]:
@@ -453,6 +627,64 @@ class Tim:
     id = self.create_anomaly_detection(parent_job_id=parent_job_id, job_configuration=job_configuration)
     return self.execute_anomaly_detection(id, wait_to_finish, handle_status_poll)
 
+  def rebuild_anomaly_detection_model(
+      self, parent_job_id: str, job_configuration: Optional[AnomalyDetectionRebuildModelConfiguration] = None
+  ) -> str:
+    """Create an anomaly detection job to rebuild the model(s) of the parent anomaly detection job
+
+    Parameters
+    ----------
+    parent_job_id : str
+        The ID of a parent anomaly detection job
+    job_configuration : AnomalyDetectionRebuildModelConfiguration, Optional
+        TIM Engine model rebuilding and detection configuration, by default None
+        Available keys are: name, configuration, data
+
+    Returns
+    -------
+    id : str
+    """
+    return rebuild_anomaly_detection_model(
+        credentials=self.credentials, parent_job_id=parent_job_id, job_configuration=job_configuration
+    )['id']
+
+  def rebuild_anomaly_detection_model_and_execute(
+      self,
+      parent_job_id: str,
+      job_configuration: Optional[AnomalyDetectionRebuildModelConfiguration] = None,
+      wait_to_finish: bool = True,
+      handle_status_poll: Optional[Callable[[StatusResponse], None]] = None
+  ) -> Union[AnomalyDetectionResultsResponse, ExecuteResponse]:
+    """Create an anomaly detection job to rebuild the model(s) of the parent anomaly detection job and execute it
+
+    Parameters
+    ----------
+    parent_job_id : str
+        The ID of a parent anomaly detection job
+    job_configuration : AnomalyDetectionRebuildModelConfiguration, Optional
+        TIM Engine model rebuilding and anomaly detection configuration, by default None
+        Available keys are: name, configuration, data
+    wait_to_finish : bool, Optional
+      Wait for all results to be calculated before returning
+      If set to False, the function will return once the job has started the execution process
+    handle_status_poll: Callable, Optional
+      A callback function to poll for the status and progress of the anomaly detection job execution
+
+    Returns
+    -------
+    metadata : Dict | None
+      Dict when successful; None when unsuccessful
+    model_result : Dict | None
+      Dict when successful; None when unsuccessful
+    table_result : DataFrame | None
+      DataFrame when successful; None when unsuccessful
+    logs : list of Dict
+    """
+    id = self.rebuild_anomaly_detection_model(
+        parent_job_id=parent_job_id, job_configuration=job_configuration
+    )
+    return self.execute_anomaly_detection(id, wait_to_finish, handle_status_poll)
+
   def get_anomaly_detection_results(self, anomaly_detection_job_id: str) -> AnomalyDetectionResultsResponse:
     """Retrieve the results of an anomaly detection job
 
@@ -477,10 +709,10 @@ class Tim:
 
     if Status(status['status']).value != Status.FAILED.value:
       metadata = get_anomaly_detection(self.__credentials, anomaly_detection_job_id)
+      table_result = get_anomaly_detection_table_results(self.__credentials, anomaly_detection_job_id)
       is_detect_job = AnomalyDetectionType(metadata['type']).value == AnomalyDetectionType.DETECT.value
       job_id = metadata['parentJob']['id'] if is_detect_job else anomaly_detection_job_id
       model_result = get_anomaly_detection_model_results(self.__credentials, job_id)
-      table_result = get_anomaly_detection_table_results(self.__credentials, job_id)
 
     return AnomalyDetectionResultsResponse(metadata, model_result, table_result, logs)
 
@@ -502,8 +734,8 @@ class Tim:
       self,
       offset: int = 0,
       limit: int = 10000,
-      user_group_id: Union[str, None] = None,
-      sort: Union[str, None] = None
+      user_group_id: Optional[str] = None,
+      sort: Optional[str] = None
   ) -> List[Workspace]:
     """Get a list of workspaces and their metadata
 
@@ -513,9 +745,9 @@ class Tim:
         Number of records to be skipped from beginning of the list, by default 0
     limit : int, optional
         Maximum number of records to be returned, by default 10000
-    user_group_id : Union[str, None], optional
+    user_group_id : Optional[str], optional
         User Group ID, by default None
-    sort : Union[str, None], optional
+    sort : Optional[str], optional
         Sorting output by the chosen attribute. +/- indicates ascending/descending order, by default -createdAt
         Available values : +createdAt, -createdAt, +updatedAt, -updatedAt, +title, -title
 
@@ -544,8 +776,8 @@ class Tim:
       self,
       offset: int = 0,
       limit: int = 10000,
-      workspace_id: Union[str, None] = None,
-      sort: Union[str, None] = None
+      workspace_id: Optional[str] = None,
+      sort: Optional[str] = None
   ) -> List[Dataset]:
     """Get a list of datasets and their metadata
 
@@ -555,9 +787,9 @@ class Tim:
         Number of records to be skipped from beginning of the list, by default 0
     limit : int, optional
         Maximum number of records to be returned, by default 10000
-    workspace_id : Union[str, None] = None
+    workspace_id : Optional[str] = None
         Filter for specific Workspace, by default None
-    sort : Union[str, None] = None
+    sort : Optional[str] = None
         Sorting output by the chosen attribute. +/- indicates ascending/descending order.
         Available values : +createdAt, -createdAt
 
@@ -598,16 +830,16 @@ class Tim:
 
   def get_anomaly_detection_jobs(
       self,
-      offset: Union[int, None] = None,
-      limit: Union[int, None] = None,
+      offset: Optional[int] = None,
+      limit: Optional[int] = None,
       sort: str = '-createdAt',
-      experiment_id: Union[str, None] = None,
-      use_case_id: Union[str, None] = None,
-      type: Union[str, None] = None,
-      status: Union[str, None] = None,
-      parent_id: Union[str, None] = None,
-      from_datetime: Union[str, None] = None,
-      to_datetime: Union[str, None] = None
+      experiment_id: Optional[str] = None,
+      use_case_id: Optional[str] = None,
+      type: Optional[str] = None,
+      status: Optional[str] = None,
+      parent_id: Optional[str] = None,
+      from_datetime: Optional[str] = None,
+      to_datetime: Optional[str] = None
   ) -> List[AnomalyDetection]:
     """Get a list of all anomaly detection jobs and their metadata
 
@@ -620,21 +852,21 @@ class Tim:
     sort : str, optional
         Sorting output by the chosen attribute. +/- indicates ascending/descending order, by default '-createdAt'
         Available values : +createdAt, -createdAt, +executedAt, -executedAt, +completedAt, -completedAt, +priority, -priority
-    experiment_id : Union[str, None], optional
+    experiment_id : Optional[str], optional
         Filter for a specific Experiment, by default None
-    use_case_id : Union[str, None], optional
+    use_case_id : Optional[str], optional
         Filter for a specific Use Case, by default None
-    type : Union[str, None], optional
+    type : Optional[str], optional
         Filter for specific types (comma separated string), by default None
         Available values : build-model, rebuild-model, detect, rca
-    status : Union[str, None], optional
+    status : Optional[str], optional
         Filter for specific job statuses (comma separated string), by default None
         Available values : Registered, Queued, Running, Finished, FinishedWithWarning, Failed
-    parent_id : Union[str, None], optional
+    parent_id : Optional[str], optional
         Filter for a specific parent job, by default None
-    from_datetime : Union[str, None], optional
+    from_datetime : Optional[str], optional
         Filter for a minimal date and time of job creation, by default None
-    to_datetime : Union[str, None], optional
+    to_datetime : Optional[str], optional
         Filter for a maximal date and time of job creation, by default None
 
     Returns
@@ -655,16 +887,16 @@ class Tim:
 
   def get_forecasting_jobs(
       self,
-      offset: Union[int, None] = None,
-      limit: Union[int, None] = None,
-      sort: Union[str, None] = None,
-      experiment_id: Union[str, None] = None,
-      use_case_id: Union[str, None] = None,
-      type: Union[str, None] = None,
-      status: Union[str, None] = None,
-      parent_id: Union[str, None] = None,
-      from_datetime: Union[str, None] = None,
-      to_datetime: Union[str, None] = None
+      offset: Optional[int] = None,
+      limit: Optional[int] = None,
+      sort: Optional[str] = None,
+      experiment_id: Optional[str] = None,
+      use_case_id: Optional[str] = None,
+      type: Optional[str] = None,
+      status: Optional[str] = None,
+      parent_id: Optional[str] = None,
+      from_datetime: Optional[str] = None,
+      to_datetime: Optional[str] = None
   ) -> List[ForecastMetadata]:
     """Get a list of all forecasting jobs and their metadata
 
@@ -677,21 +909,21 @@ class Tim:
     sort : str, optional
         Sorting output by the chosen attribute. +/- indicates ascending/descending order, by default '-createdAt'
         Available values : +createdAt, -createdAt, +executedAt, -executedAt, +completedAt, -completedAt, +priority, -priority
-    experiment_id : Union[str, None], optional
+    experiment_id : Optional[str], optional
         Filter for a specific Experiment, by default None
-    use_case_id : Union[str, None], optional
+    use_case_id : Optional[str], optional
         Filter for a specific Use Case, by default None
-    type : Union[str, None], optional
+    type : Optional[str], optional
         Filter for specific types (comma separated string), by default None
         Available values : build-model, rebuild-model, predict, rca
-    status : Union[str, None], optional
+    status : Optional[str], optional
         Filter for specific job statuses (comma separated string), by default None
         Available values : Registered, Queued, Running, Finished, FinishedWithWarning, Failed
-    parent_id : Union[str, None], optional
+    parent_id : Optional[str], optional
         Filter for a specific parent job, by default None
-    from_datetime : Union[str, None], optional
+    from_datetime : Optional[str], optional
         Filter for a minimal date and time of job creation, by default None
-    to_datetime : Union[str, None], optional
+    to_datetime : Optional[str], optional
         Filter for a maximal date and time of job creation, by default None
 
     Returns
